/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.monitor.ri.server;

import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import javax.swing.event.EventListenerList;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.container.PropertyException;
import org.jdtaus.core.monitor.Task;
import org.jdtaus.core.monitor.TaskEvent;
import org.jdtaus.core.monitor.TaskListener;
import org.jdtaus.core.logging.spi.Logger;
import org.jdtaus.core.monitor.spi.TaskMonitor;

/**
 * jDTAUS Core SPI Server {@code TaskMonitor} implementation.
 * <p>The server implementation will measure the time a task is running. The
 * configuration property {@code longRunningMillis} holds the amount of
 * milliseconds a task needs to run before it is considered a long running task.
 * The server implementation will then log notification messages about long
 * running tasks. Since this implementation does not monitor a task for changes
 * no {@code TaskEvent}s of type {@code CHANGED_STATE} will be fired.</p>
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: ServerTaskMonitor.java 3214 2007-04-29 22:54:13Z schulte2005 $
 *
 * @see org.jdtaus.core.container.Container
 */
public final class ServerTaskMonitor
    implements TaskMonitor, ContainerInitializer
{

    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(ServerTaskMonitor.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Protected <code>ServerTaskMonitor</code> implementation constructor.
     *
     * @param meta Implementation meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    protected ServerTaskMonitor(final Implementation meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }
    /**
     * Protected <code>ServerTaskMonitor</code> dependency constructor.
     *
     * @param meta dependency meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    protected ServerTaskMonitor(final Dependency meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

        p = meta.getProperty("longRunningMillis");
        this._longRunningMillis = ((java.lang.Long) p.getValue()).longValue();

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /**
     * Initializes the instance.
     *
     * @see #assertValidProperties()
     */
    public void initialize()
    {
        this.assertValidProperties();
    }

    //----------------------------------------------------ContainerInitializer--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Property {@code longRunningMillis}.
     * @serial
     */
    private long _longRunningMillis;

    /**
     * Gets the value of property <code>longRunningMillis</code>.
     *
     * @return the value of property <code>longRunningMillis</code>.
     */
    protected long getLongRunningMillis()
    {
        return this._longRunningMillis;
    }


    //--------------------------------------------------------------Properties--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>Logger</code> implementation. */
    private transient Logger _dependency0;

    /**
     * Gets the configured <code>Logger</code> implementation.
     *
     * @return the configured <code>Logger</code> implementation.
     */
    private Logger getLogger()
    {
        Logger ret = null;
        if(this._dependency0 != null)
        {
            ret = this._dependency0;
        }
        else
        {
            ret = (Logger) ContainerFactory.getContainer().
                getDependency(ServerTaskMonitor.class,
                "Logger");

            if(ModelFactory.getModel().getModules().
                getImplementation(ServerTaskMonitor.class.getName()).
                getDependencies().getDependency("Logger").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--TaskEventSource---------------------------------------------------------

    /** List of {@code TaskListener}s. */
    private final EventListenerList taskListeners = new EventListenerList();

    public void addTaskListener(final TaskListener listener)
    {
        if(listener == null)
        {
            throw new NullPointerException("listener");
        }

        this.taskListeners.add(TaskListener.class, listener);
    }

    public void removeTaskListener(final TaskListener listener)
    {
        if(listener == null)
        {
            throw new NullPointerException("listener");
        }

        this.taskListeners.remove(TaskListener.class, listener);
    }

    public TaskListener[] getTaskListeners()
    {
        return (TaskListener[]) this.taskListeners.
            getListeners(TaskListener.class);

    }

    //---------------------------------------------------------TaskEventSource--
    //--TaskMonitor-------------------------------------------------------------

    /** Maps {@code Task}s to theire starting timestamp. */
    private final Map taskMap = new HashMap(100);

    public void monitor(final Task task)
    {
        if(task == null)
        {
            throw new NullPointerException("task");
        }

        synchronized(this.taskMap)
        {
            this.finish(task);
            this.taskMap.put(task, new Long(System.currentTimeMillis()));
            this.fireTaskEvent(new TaskEvent(task, TaskEvent.STARTED));
        }
    }

    public void finish(final Task task)
    {
        if(task == null)
        {
            throw new NullPointerException("task");
        }

        synchronized(this.taskMap)
        {
            final Long start = (Long) this.taskMap.remove(task);
            if(start != null)
            {
                if(System.currentTimeMillis() - start.longValue() >
                    this.getLongRunningMillis())
                {

                    this.getLogger().warn(ServerTaskMonitorBundle.
                        getLongRunningTaskNotificationMessage(
                        Locale.getDefault()).format( new Object[] {

                        task.getDescription().getText(Locale.getDefault()),
                        new Date(start.longValue()),
                        new Date(System.currentTimeMillis())

                    }));
                }

                this.fireTaskEvent(new TaskEvent(task, TaskEvent.ENDED));
            }
        }
    }

    //-------------------------------------------------------------TaskMonitor--
    //--ServerTaskMonitor-------------------------------------------------------

    /** Creates a new {@code ServerTaskMonitor} instance. */
    public ServerTaskMonitor()
    {
        this(ServerTaskMonitor.META);
        this.initialize();
    }

    /**
     * Checks configured properties.
     *
     * @throws PropertyException for illegal property values.
     */
    protected void assertValidProperties()
    {
        if(this.getLongRunningMillis() < 0L)
        {
            throw new PropertyException("longRunningMillies",
                Long.toString(this.getLongRunningMillis()));

        }
    }

    /**
     * Notifies all registered {@code TaskListener}s about {@code TaskEvent}s.
     *
     * @param e The event to be provided to the listeners.
     */
    protected void fireTaskEvent(final TaskEvent e)
    {
        if(e == null)
        {
            throw new NullPointerException("e");
        }

        final Object[] listeners = this.taskListeners.getListenerList();
        for (int i = listeners.length - 2; i >= 0; i -= 2)
        {
            if (listeners[i] == TaskListener.class)
            {
                ((TaskListener) listeners[i + 1]).onTaskEvent(e);
            }
        }
    }

    //-------------------------------------------------------ServerTaskMonitor--

}
